@@ -18,7 +18,8 @@ import re
 import sys
 import importlib
 from utils.text import convertStrListDict, _toUNICODE, formatListOr
-from utils.files import pathJoin
+from utils.files import pathJoin,readFile
+from utils.portage import searchProfile
 from os import path
 import os
 from collections import OrderedDict
@@ -30,6 +31,13 @@ import operator
 import traceback
 from types import StringTypes
 
+def addStdConfig(fileLst,prefix='/'):
+    stdPath = path.join(prefix,
+              "var/lib/layman/calculate/profiles/calculate.env")
+    if not stdPath in map(os.path.abspath,fileLst):
+            return  [stdPath] + fileLst
+    return fileLst
+
 def itemgetter(*args,**kwargs):
     """
     itemgetter with alwaysTuple param, which used for force tuple elements
@@ -96,6 +104,8 @@ class Variable:
     name = ""
     # gui label
     label = None
+    # syntax for opts
+    syntax = ""
     # opt for console
     opt = None
     # help for console
@@ -120,16 +130,16 @@ class Variable:
         Initialize instance
         """
         self.modeGet = Variable.NONE
-        self.invalid = self.value is None
         self.wasSet = False     # variable is user or ini set
         self.reqVars = []       # varaibles which used this value for fill
         self.reqCheck = []      # variables which used this value for check
         self.reqUncompat = []   # varaibles which used for uncompat
         self.variableType = type
         self.parent = parent
+        self.invalid = True # self.value is None or self.name in self.parent.iniCache
         self.name = self.getVariableName()
-        self.init()
         self.processInvalidate = False
+        self.init()
         for key,val in kwargs.items():
             setattr(self,key,val)
 
@@ -143,6 +153,18 @@ class Variable:
     def getVariableName(cls):
         return re.sub("(.)([A-Z])","\\1_\\2",cls.__name__[8:]).lower()
 
+    def getCharType(self):
+        """
+        Get symbol of type
+        """
+        mapType = OrderedDict(table='t',list='l',choice='c',number='i',
+                              bool='b') 
+        for k in mapType.keys():
+            if k in self.type:
+                return mapType[k]
+        else:
+            return "s"
+
     def Get(self,varname=None,humanreadable=False):
         """Get value of other value"""
         if not varname or varname == self.name:
@@ -150,6 +172,8 @@ class Variable:
 
         varObj = self.parent.getInfo(varname)
         #varObj.modeGet = Variable.UNCOMPAT
+        if not "." in varname:
+            varname = "%s.%s"%(self.section,varname)
         res = self.parent.Get(varname,humanreadable)
 
         # build dependence for check and uncompatible
@@ -221,6 +245,25 @@ class Variable:
         else:
             return self._get()
 
+    def iniread(self):
+        """
+        Read from ini file
+        """
+        value = self.parent.iniCache[self.name]['value']
+        section = self.parent.iniCache[self.name]['section']
+        value = self.parent.unserialize(self.type,
+                                 value.encode('utf-8'))
+        try:
+            if self.mode == READONLY:
+                raise VariableError(
+                    _("Attempt to rewrite readonly variable %s")%
+                    varname)
+            else:
+                self._set(value)
+        except Exception as e:
+            return False
+        return True
+
     def _get(self):
         """
         Standard inner method for getting value of variable.
@@ -228,12 +271,16 @@ class Variable:
         # variable need refill
         if self.invalid:
             # get net value
-            try:
-                oldModeGet = self.modeGet
-                self.modeGet = Variable.NONE
-                value = self.get()
-            finally:
-                self.modeGet = oldModeGet
+            if self.name in self.parent.iniCache and \
+                self.iniread():
+                return self.value
+            else:
+                try:
+                    oldModeGet = self.modeGet
+                    self.modeGet = Variable.NONE
+                    value = self.get()
+                finally:
+                    self.modeGet = oldModeGet
             # if value not equal previous value
             if value != self.value:
                 # invalidate depended variables
@@ -308,6 +355,12 @@ class Variable:
             return map(convert,value)
         return value
 
+    def raiseNothingValue(self):
+        """
+        Using for override error message about nothing value.
+        """
+        pass
+
     def checkType(self,value):
         """Check value for type"""
         if "list" in self.type:
@@ -339,7 +392,12 @@ class Variable:
                     if choiceVal and val and not val in choiceVal or \
                         val is None:
                         name = self.label or self.name
-                        raise VariableError(error.format(
+                        if not choiceVal:
+                            self.raiseNothingValue()
+                        self.raiseWrongChoice(name,choiceVal,val,error)
+
+    def raiseWrongChoice(self,name,choiceVal,value,error):
+        raise VariableError(error.format(
                             varname=name,
                             vartype=formatListOr(choiceVal)))
 
@@ -432,7 +490,7 @@ class TableVariable(Variable):
                              imap(lambda x:(x,self.Get(x)),
                              self.source)):
             raise VariableError(
-                _("Source variable %s not contains list")%varname)
+                _("Source variable %s does not contain a list")%varname)
         return map(list,
                izip_longest(
                *map(lambda x:self.Get(x,humanreadable=hr),
@@ -495,20 +553,31 @@ class TableVariable(Variable):
         writeLen = len(self.__getWritableColumns())+1
         for item in value:
             if item and not len(item) in (writeLen,len(self.source)):
-                raise DataVarsError(
+                raise VariableError(
                     _("Wrong entry '{entry}' for table variable '{varname}'").
                     format(entry=str(item),
-                    varname=(self.label or self.name).lower()))
+                    varname=(self.label or self.name).lower())+"\n"+
+                    _("Each entry must contains %d field(s)")%writeLen)
 
         # check rewrite readonly index
         if not self.__isIndexWritable():
             indexValues = self.Get(self.source[0])
             for item in value:
                 if item and not item[0] in indexValues:
-                    raise DataVarsError(
-                        _("Attempt to rewrite a readonly index field "
-                        "{fieldname} in variable {variablename}").format(
-                        fieldname=self.source[0],variablename=self.name))
+                    self.raiseReadonlyIndexError(self.source[0],
+                                                 self.name,
+                                                 item[0])
+
+    def raiseReadonlyIndexError(self,fieldname="",variablename="",
+                                     value=""):
+        """
+        Overriddible method for spcifing behavior on rewrite
+        readonly index
+        """
+        raise DataVarsError(
+            _("Attempt to rewrite a readonly index field "
+            "{fieldname} in variable {variablename}").format(
+            fieldname=fieldname,variablename=variablename))
 
     def setValue(self,value):
         value = self.set(value)
@@ -568,6 +637,18 @@ class ReadonlyVariable(Variable):
     """
     mode = READONLY
 
+class ActionVariable(ReadonlyVariable):
+    nonchroot = False
+    postinst = False
+
+    def get(self):
+        if self.nonchroot and self.Get('cl_chroot_status') == "on":
+            return "off"
+        if self.postinst and not self.Get('cl_ebuild_phase') in \
+            ('','postinst','prerm'):
+            return "off"
+        return self.action(self.Get('cl_action'))
+
 class FieldValue:
     """
     Table column variable
@@ -647,6 +728,7 @@ class SimpleDataVars:
     """
     Simple datavars for temporary link var objs.
     """
+    systemRoot = "/"
     def __init__(self,*objs):
         self.allVars = {}
         self.cache = {}
@@ -655,6 +737,7 @@ class SimpleDataVars:
             self.allVars[obj.getVariableName()] = obj
 
     def Get(self,varname,humanreadable=False):
+        varname = varname.rpartition(".")[2]
         if not varname in self.cache:
             self.cache[varname] = self.allVars.get(varname).get()
         return self.cache[varname]
@@ -667,14 +750,50 @@ class SimpleDataVars:
         Read variable values from ini files.
         Return False if ini file is invalid.
         """
-        config = cl_template.iniParser(iniFile)
+        if path.isdir(iniFile):
+            config = cl_template.iniParser(
+                text=reduce(lambda x,y:x+readFile(y),
+                addStdConfig(
+                searchProfile(iniFile,'calculate.env'),prefix=self.systemRoot),""))
+        else:
+            config = cl_template.iniParser(iniFile)
+        importVars = config.getAreaVars("main")
         for varname in self.allVars.keys():
             if not varname in self.cache:
-                val = config.getVar('main',varname)
+                val = self.unserialize("string",importVars.get(varname,''))
                 if val:
                     self.cache[varname] = val.encode('utf-8')
 
-class DataVars:
+    def serialize(self,varType,value):
+        """
+        Serialize to string for ini file
+        """
+        fixEmpty = lambda x: str(x) or "''"
+        fixSpace = lambda x: "'%s'"%str(x) if " " in str(x) else str(x)
+        isListOrTuple = lambda x: type(x) in (list,tuple)
+        if isListOrTuple(value):
+            if "list" in varType:
+                return ",".join(map(fixEmpty,value))
+            elif "table" in varType:
+                return ";".join(map(lambda x:",".join(map(fixEmpty,x)),
+                                value))
+        return fixEmpty(value)
+
+    def unserialize(self,varType,value):
+        """
+        Unserialize form string for varname
+        """
+        fixEmpty = lambda x:"" if x=="''" else x
+        def getList(val):
+            return map(fixEmpty,value.split(','))
+        if "list" in varType:
+            return getList(value)
+        if "table" in varType:
+            return map(getList,value.split(';'))
+        return fixEmpty(value).strip("'")
+
+
+class DataVars(SimpleDataVars):
     """
     Class contains variables, perform creating variable objects by
     class variable describer and variable filler.
@@ -690,6 +809,7 @@ class DataVars:
         dependence of variables.
     """
     defaultData = 'calculate.lib.variables'
+    reRightVar = re.compile(r'^(?:([a-z_]+)\.)?([a-z_0-9]+)$')
     l = RLock()
 
     def __init__(self):
@@ -701,8 +821,27 @@ class DataVars:
         self.groups = []
         self.mapVarGroup = OrderedDict()
         self.briefData = {}
+        self.filledVars = {}
+        self.defaultModule = "main"
+        self.importedModules = []
+        self.iniCache = {}
+
+    def isModuleInstalled(self,module):
+        try:
+            importlib.import_module("calculate.%s.datavars"%module)
+            return True
+        except (ImportError) as e:
+            return False
 
     def importData(self,data=None):
+        """
+        Depricated function for support 3.0 utilities
+        """
+        section = self.importVariables(data)
+        if data:
+            self.defaultModule = section
+
+    def importVariables(self,data=None):
         """
         Import modules for variables. 'section' is section name, 'modList'
         contains tuple with modules which discribe variable names and fill
@@ -714,9 +853,8 @@ class DataVars:
         try:
             varModule = importlib.import_module(data)
         except (ImportError,AttributeError),e:
-            raise DataVarsError("\n".join([
-                _("Failed to import module %s")%data,
-                _("error")  + ": " +str(e)]))
+            if not self.raiseModuleError(data,e):
+                return False
         # get all Variable classes from module
         # variable of class VariableClAction will be named cl_action
         if hasattr(varModule,"section"):
@@ -734,38 +872,49 @@ class DataVars:
                     dir(varMod))):
                 varObj = getattr(varMod,classname)
                 varName = varObj.getVariableName()
-                self.allVars[varName] = (section,varObj)
+                if not varName in self.allVars:
+                    self.allVars[varName] = (section,varObj)
+        self.importedModules.append(section)
+        return section
 
-    def stubVariable(self,varname,*args,**kwargs):
+    def raiseVariableNotFound(self,varname,*args,**kwargs):
         raise DataVarsError(_("Variable %s not found")%varname)
 
-    def loadVariable(self,varname):
+    def raiseModuleError(self,module,error,*args,**kwargs):
+        raise DataVarsError("\n".join([
+            _("Failed to import module %s")%module,
+            _("error")  + ": " +str(error)]))
+
+    def loadVariable(self,varname,section=None):
         """
         Initialize variable 'varname' from fillObj and varObj.
         """
         if not self.allVars:
-            self.importData()
+            self.importVariables()
+        if section and not section in self.importedModules:
+            self.importVariables("calculate.%s.variables"%section)
+            self.flIniFile(onlySection=section)
         if not varname in self.loadVariables:
             if varname in self.allVars.keys():
                 section,varObj = self.allVars[varname]
                 newVar = varObj(parent=self,section=section)
                 if hasattr(newVar,"refresh"):
                     self.refresh.append(newVar)
-                #attrs = (("Fill","get_%s"),
-                #         ("Check","check_%s"),
-                #         ("Choice","choice_%s"),
-                #         ("Uncompatible","uncompatible_%s"),
-                #         ("HumanRead","human_%s"),
-                #         ("Replace","set_%s"))
-                #for attrname,funcname in attrs:
-                #    funcname = funcname%varname
-                #    if hasattr(varObj,funcname):
-                #        setattr(newVar,attrname,getattr(varObj,funcname))
-                #setattr(newVar,"Get",self.Get)
                 self.loadVariables[varname] = newVar
             else:
+                # this code using for raise exception or
+                # if replace raiseVariableNotFound method
+                # then return default value
                 self.loadVariables[varname] = \
-                    self.stubVariable(varname,parent=self,section="main")
+                    self.raiseVariableNotFound(
+                        varname,parent=self,section="main")
+
+    def exists(self,varName):
+        try:
+            self.Get(varName)
+            return True
+        except:
+            return False
 
     def defined(self,varName):
         """
@@ -783,7 +932,7 @@ class DataVars:
         self.getIniVar('install.os_install_net_settings')
         self.getIniVar('cl_action')
         """
-        calculateIniFiles = self.Get('cl_env_path')
+        calculateIniFiles = self.Get('main.cl_env_path')
         section, dot, varname = fullVarName.rpartition('.')
         if not section:
             section = "main"
@@ -826,49 +975,45 @@ class DataVars:
                     optionsInfo[section] = options
         return optionsInfo
 
-    def flIniFile(self,raiseOnError=False):
+    def flIniFile(self,raiseOnError=False,onlySection=None):
         """
         Read variable values from ini files.
         Return False if ini file is invalid.
         """
-        calculateIniFiles = self.Get('cl_env_path')
-        # get initialized section names
-        sections = set(map(lambda x:x[0],self.allVars.values()))
-        res = True
-        for iniFile in calculateIniFiles:
-            if path.exists(iniFile):
-                config = cl_template.iniParser(iniFile)
-                iniSections = config.getAllSectionNames()
-                if not iniSections:
-                    continue
-                iniSections = tuple(set(iniSections)&sections)
-                for section in iniSections:
-                    importVars = config.getAreaVars(section)
-                    if self.allVars == False:
-                        return False
-                    for key,value in importVars.items():
-                        try:
-                            key = _toUNICODE(key)
-                            origkey = key.encode('utf-8')
-                            value = convertStrListDict(_toUNICODE(value))
-                            if not key in self.loadVariables:
-                                self.loadVariable(origkey)#,onlySection=section)
-                            varObj = self.loadVariables[origkey]
-                            if "list" in varObj.type and \
-                                not type(value) in (list,tuple) :
-                                value = map(lambda x:x.strip().encode('utf-8'),
-                                        value.split(','))
-                            self.Set(origkey,value)
-                        except Exception as e:
-                            res = False
-                            if raiseOnError:
-                                raise
-                            else:
-                                sys.stdout.write(
-                                    _("Error reading the ini file: "))
-                                sys.stdout.write(str(e)+"\n")
-                                sys.stdout.flush()
-        return res
+        if not self.iniCache:
+            # get initialized section names
+            if os.path.exists('/etc/make.profile'):
+                profiles = [('profile',
+                            path.join('/etc',os.readlink('/etc/make.profile')))]
+            else:
+                profiles = []
+            for iniName,iniFile in profiles+\
+                                    self.Get('main.cl_env_data'):
+                if path.exists(iniFile) or iniName == 'profile':
+                    if iniName == 'profile':
+                        config = cl_template.iniParser(
+                            text=reduce(lambda x,y:x+readFile(y),
+                            addStdConfig(
+                            searchProfile(iniFile,'calculate.env')),""))
+                    else:
+                        config = cl_template.iniParser(iniFile)
+                    iniSections = config.getAllSectionNames()
+                    if not iniSections:
+                        continue
+                    if onlySection:
+                        iniSections = tuple(set(iniSections)&
+                                            set([unicode(onlySection)]))
+                    for section in iniSections:
+                        importVars = config.getAreaVars(section)
+                        if self.allVars == False:
+                            continue
+                        for key,value in importVars.items():
+                            key = _toUNICODE(key).encode('utf-8')
+                            self.iniCache[key] = {'value':value,
+                                              'section':section}
+                            if iniName != "profile":
+                                self.filledVars[key] = iniName
+        return True
 
     def Get(self,varname,humanreadable=False):
         """Threading safety Get"""
@@ -876,13 +1021,29 @@ class DataVars:
         try:
             var = self.__Get(varname,humanreadable)
         finally:
-            pass
             DataVars.l.release()
         return var
 
     def GetBool(self,varname):
         return Variable.isTrue(self.Get(varname))
 
+    def isFromIni(self,varname):
+        """
+        Check what value get from ini file
+        """
+        section,varname = self.splitVarname(varname)
+        return self.filledVars[varname] if varname  in self.filledVars else ""
+
+    def splitVarname(self,varname):
+        res = self.reRightVar.search(varname)
+        if res:
+            if res.group(1):
+                return res.groups()
+            else:
+                return (self.defaultModule,res.group(2))
+        else:
+            raise DataVarsError(_("Wrong variable name %s")%varname)
+
     def __Get(self, varname, humanreadable=False):
         """
         Get value of variable 'varname'
@@ -890,9 +1051,16 @@ class DataVars:
         try:
             #tm = time.time()
             # load variable if it isn't loaded
-            if not varname in self.loadVariables:
-                self.loadVariable(varname)
+            section,varname = self.splitVarname(varname)
+            if not varname in self.loadVariables or \
+                not section in self.importedModules:
+                self.loadVariable(varname,section=section)
             varObj = self.loadVariables[varname]
+
+            # check section
+            if not varObj.section in (section,"main"):
+                self.raiseVariableNotFound(varname,parent=self,section=section)
+
             # if for this variable already use Get method
             if varObj.invalid and varObj in self.requestVariables:
                 varnames = "-".join(map(lambda x:x.name,
@@ -978,8 +1146,9 @@ class DataVars:
         """
         Get info (label and other for variable)
         """
+        section,varname = self.splitVarname(varname)
         if not varname in self.loadVariables:
-            self.loadVariable(varname)
+            self.loadVariable(varname,section=section or None)
         return self.loadVariables[varname]
 
     def Set(self, varname, varvalue,force=False):
@@ -991,7 +1160,7 @@ class DataVars:
             varObj._set(varvalue)
         else:
             raise DataVarsError(
-                _("Attempt to rewrite a readonly variable %s")%
+                _("Attempt to rewrite readonly variable %s")%
                 varname)
 
     def Invalidate(self,varname,force=False,onlySet=False):
@@ -1014,16 +1183,16 @@ class DataVars:
                                        where='cl_env_location',
                                        eq=location,limit=1)
         if calculateIniFile:
-            return pathJoin(self.Get('cl_chroot_path'),calculateIniFile)
+            return pathJoin(self.Get('main.cl_chroot_path'),calculateIniFile)
         else:
             return False
 
-    def Write(self, varname, val=None, force=False, location='default',
+    def Write(self, varname, val=None, force=False, location='system',
               header=False):
         """
         Write variable 'varname' to ini file. If set 'val', then to
         variable before write perform value changing. 'force' use for
-        change value of read-only variables. 'location' must be ('default',
+        change value of read-only variables. 'location' must be ('system',
         'local', 'remote') use for specify type of ini file location ini.
         'header' use for specify section in ini file. If header not
         specified then use default for this value section.
@@ -1031,7 +1200,10 @@ class DataVars:
         # set value to variable if specified value
         if not val is None:
             self.Set(varname,val,force=force)
-        val = self.Get(varname)
+        else:
+            if header:
+                vname = "%s.%s"%(header,varname)
+            val = self.Get(vname)
         # get path to target ini file
         iniFile = self.getEnvFileByLocation(location)
         if iniFile:
@@ -1042,20 +1214,23 @@ class DataVars:
                 header = self.loadVariables[varname].section
             # write value to ini file
             config = cl_template.iniParser(iniFile)
-            if "list" in self.getInfo(varname).type:
-                val = ",".join(val)
-            if not config.setVar(header,{varname: convertStrListDict(val)}):
-                raise DataVarsError(_("Unable to write variable %s")%varname)
+            val = self.serialize(self.getInfo(varname).type,
+                  val)
+            writeVal = convertStrListDict(val)
+            if writeVal != config.getVar(header,varname):
+                if not config.setVar(header,{varname: writeVal}):
+                    raise DataVarsError(
+                        _("Unable to write variable %s")%varname)
             return True
         else:
-            raise DataVarsError(_("Unable to find alias '%s' of the path"
-                  " for storage of template variables")%location)
+            raise DataVarsError(_("Unable to find alias '%s' for the "
+                                  "template variables path")%location)
         return False
 
-    def Delete(self, varname, location='default', header=False):
+    def Delete(self, varname, location='system', header=False):
         """
         Delete variable 'varname' from ini file and refresh value.
-        'location' must be ('default', 'local', 'remote') use for specify
+        'location' must be ('system', 'local', 'remote') use for specify
         type of ini file location ini. 'header' use for specify section in
         ini file. If header not specified then use default for this value
         section.
@@ -1067,6 +1242,8 @@ class DataVars:
             makePath(path.dirname(iniFile))
             # get variable section if not specified header
             if not header:
+                if not varname in self.loadVariables:
+                    self.loadVariable(varname)
                 header = self.loadVariables[varname].section
             # write value to ini file
             config = cl_template.iniParser(iniFile)
@@ -1081,8 +1258,8 @@ class DataVars:
                                       config.delArea(header))
             return True
         else:
-            raise DataVarsError(_("Unable to find alias '%s' of the path"
-                  " for storage of template variables")%location)
+            raise DataVarsError(_("Unable to find alias '%s' for the "
+                                  "template variables path")%location)
         return False
 
     def Select(self,selField,where="os_disk_dev",eq=None,ne=None,
@@ -1241,6 +1418,17 @@ class DataVars:
                 val = list(izip_longest(*val,fillvalue=""))
         return val
 
+    def plainList(self,*elements):
+        """
+        Make plain list from many dimensions
+        """
+        for w in elements:
+            if type(w.message) in (list,tuple):
+                for i in self.plainList(*w.message):
+                    yield i
+            else:
+                yield w
+
     def checkGroups(self,info,allvars=False):
         """
         Check variables in group or all in info
@@ -1298,11 +1486,11 @@ class DataVars:
                         # raise error for atempt set uncompatible variable
                         if uncomperr:
                             raise VariableError(uncomperr)
-                except VariableError as e:
+                except (VariableError,DataVarsError) as e:
                     # assemble all variable errors
                     messages = e.message if type(e.message) == list \
-                               else [e.message]
-                    mess = "\n".join(map(lambda x:str(x),messages))
+                               else [e]
+                    mess = "\n".join(map(lambda x:str(x),self.plainList(*messages)))
                     if isinstance(e,PasswordError) or \
                         type(e.message) == list and \
                         all(isinstance(x,PasswordError) for x in e.message):
@@ -1312,7 +1500,7 @@ class DataVars:
                     errors.append({'type':typeError, 'field':var, 'message':mess})
                 except BaseException as e:
                     for i in apply(traceback.format_exception, sys.exc_info()):
-                        print i
+                        print i,
                     errors.append({'type':'error', 'field':var,
                                    'message':str(e)})
         return errors
