@@ -88,6 +88,7 @@ class process:
         self.iter = None
         self.pipe = None
         self.cacheresult = None
+        self.cacheerr = None
         self.communicated = False
 
     def _open(self):
@@ -124,12 +125,37 @@ class process:
         if self.pipe:
             self.pipe.stdin.close()
 
+    def readerr(self):
+        self.read()
+        return self.cacheerr
+
+    def readByLine(self):
+        
+        try:
+            self._open()
+            if self.cacheresult is None:
+                self.cacheresult = []
+                s = self.pipe.stdout.readline()
+                while s:
+                    yield s
+                    s = self.pipe.stdout.readline()
+                if self.pipe.stderr:
+                    self.cacheerr = self.pipe.stderr.read()
+                else:
+                    self.cacheerr = ""
+
+        except KeyboardInterrupt:
+            self.kill()
+            raise KeyboardInterrupt
+        finally:
+            self.cacheresult = "\n".join(self.cacheresult)
+
     def read(self):
         """Read all data"""
         try:
             self._open()
             if self.cacheresult is None:
-                self.cacheresult = self.pipe.communicate()[0]
+                self.cacheresult,self.cacheerr = self.pipe.communicate()
             return self.cacheresult
         except KeyboardInterrupt:
             self.kill()
